@@ -6,7 +6,7 @@ use POSIX qw(:errno_h);
 use Socket qw(PF_INET SOCK_STREAM pack_sockaddr_in inet_ntoa $CRLF SOL_SOCKET SO_ERROR);
 use Fcntl qw(F_GETFL F_SETFL O_NONBLOCK);
 
-our $VERSION = "0.26";
+our $VERSION = "0.27";
 
 sub Hijk::Error::CONNECT_TIMEOUT         () { 1 << 0 } # 1
 sub Hijk::Error::READ_TIMEOUT            () { 1 << 1 } # 2
@@ -78,18 +78,19 @@ sub _read_http_message {
                 my ($doing_chunked, $content_length, $trailer_mode, $trailer_value_is_true);
                 for (split /${CRLF}/o, $head) {
                     my ($key, $value) = split /: /, $_, 2;
+                    my $key_lc = lc($key);
 
                     # Figure this out now so we don't need to scan the
                     # list later under $head_as_array, and just for
                     # simplicity and to avoid duplicating code later
                     # when !$head_as_array.
-                    if ($key eq 'Transfer-Encoding' and $value eq 'chunked') {
+                    if ($key_lc eq 'transfer-encoding' and $value eq 'chunked') {
                         $doing_chunked = 1;
-                    } elsif ($key eq 'Content-Length') {
+                    } elsif ( ($key_lc eq 'content-length') || (lc($key) eq 'content-length') ) {
                         $content_length = $value;
-                    } elsif ($key eq 'Connection' and $value eq 'close') {
+                    } elsif ($key_lc eq 'connection' and $value eq 'close') {
                         $close_connection = 1;
-                    } elsif ($key eq 'Trailer' and $value) {
+                    } elsif ($key_lc eq 'trailer' and $value) {
                         $trailer_value_is_true = 1;
                     }
 
@@ -147,11 +148,11 @@ sub _read_chunked_body {
     my $chunk_size   = 0;
     my $body         = "";
     my $trailer_mode = 0;
-
+    my $wait_for_last_clrf = 0;
     vec(my $rin = '', $fd, 1) = 1;
     while(1) {
         # just read a 10k block and process it until it is consumed
-        if (length($buf) == 0 || length($buf) < $chunk_size) {
+        if (length($buf) < 3 || length($buf) < $chunk_size || $wait_for_last_clrf > 0) {
             return (undef, Hijk::Error::READ_TIMEOUT)
                 if ((_select($rin, undef, undef, $read_timeout) != 1) || (defined($read_timeout) && $read_timeout <= 0));
             my $current_buf = "";
@@ -177,6 +178,12 @@ sub _read_chunked_body {
 
             $buf .= $current_buf;
         }
+
+        if ($wait_for_last_clrf > 0) {
+            $wait_for_last_clrf -= length($buf);
+            return $body if ($wait_for_last_clrf <= 0);
+        }
+
         if ($trailer_mode) {
             # http://tools.ietf.org/html/rfc2616#section-14.40
             # http://tools.ietf.org/html/rfc2616#section-3.6.1
@@ -202,7 +209,7 @@ sub _read_chunked_body {
             }
         } else {
             if ($chunk_size > 0 && length($buf) >= $chunk_size) {
-                $body .= substr($buf, 0, $chunk_size - 2); # our chunk size includes the CRLF
+                $body .= substr($buf, 0, $chunk_size - 2); # our chunk size includes the following CRLF
                 $buf = substr($buf, $chunk_size);
                 $chunk_size = 0;
             } else {
@@ -213,12 +220,25 @@ sub _read_chunked_body {
                         if ($true_trailer_header) {
                             $trailer_mode = 1;
                         } else {
-                            return $body;
+                            $buf = substr($buf, $neck_pos + 2);
+                            # in case we are missing the ending CLRF, we have to wait for it
+                            # otherwise it is left int he socket
+                            if (length($buf) < 2) {
+                                $wait_for_last_clrf = 2 - length($buf);
+                            } else {
+                                return $body;
+                            }
                         }
                     } else {
-                        $chunk_size += 2;                  # include the final CRLF
+                        $chunk_size += 2; # include the following CRLF
                         $buf = substr($buf, $neck_pos + 2);
                     }
+                } elsif($neck_pos == 0) {
+                    return (
+                        undef,
+                        Hijk::Error::RESPONSE_BAD_READ_VALUE,
+                        "Wasn't expecting CLRF without chunk size. This shouldn't happen, buf:<$buf>",
+                    );
                 }
             }
         }
@@ -428,7 +448,7 @@ A simple GET request:
         die "Oh noes we had some sort of timeout";
     }
 
-    die "Expecting a successful response" unless $res->{status} == 200;
+    die "Expecting an 'OK' response" unless $res->{status} == 200;
 
     say $res->{body};
 
@@ -448,7 +468,7 @@ escape your values etc.
         body         => "description=" . uri_escape("Another flower, let's hope it's exciting"),
     });
 
-    die "Expecting a successful response" unless $res->{status} == 200;
+    die "Expecting an 'OK' response" unless $res->{status} == 200;
 
 =head1 DESCRIPTION
 
@@ -598,7 +618,7 @@ C<http://example.com/flower?color=red>, pass the following parameters:
         path         => "/flower",
         query_string => "color=red"
     });
-    die "Response is not OK" unless $res->{status} == 200;
+    die "Response is not 'OK'" unless $res->{status} == 200;
 
 Notice that you do not need to put the leading C<"?"> character in the
 C<query_string>. You do, however, need to properly C<uri_escape> the content of
